/**
 * @file
 *
 * getch() avances
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <conio.h>
#include <time.h>
#include "getch_ex.h"

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC	CLK_TCK 
#endif

#define MAX_TIMEOUT 32

/**
 * --code_to_char_ascii()--
 * Extrait le code ascii 
 *
 * @param code notre code
 * @return code ascii
 */
int code_to_char_ascii( long code )
{ 
    // On utilise l'oprateur binaire & pour mettre les 24 bits 
    // de poids forts  0 et ne conserver que les 8 bits de poids faible.
    return code & 0xff;
}


/**
 * --code_to_char_extended()--
 * Extrait le code etendu 
 *
 * @param code notre code
 * @return code ascii
 */
int code_to_char_extended( long code )
{
    // On utilise l'oprateur binaire >> pour dcaler le code 
    // de 8 bits vers la droite, puis on procde comme prcdemment 
    // pour mettre les 24 bits de poids forts  0 et ne 
    // conserver que les 8 bits de poids faible.
    code = code >> 8;
    return code & 0xff;
}

/**
 * --code_to_timeout_index()--
 * Extrait le code etendu 
 *
 * @param code notre code
 * @return code ascii
 */
int code_to_timeout_index( long code )
{
    // On utilise l'oprateur binaire >> pour dcaler le code 
    // de 16 bits vers la droite, puis on procde comme prcdemment 
    // pour mettre les 24 bits de poids forts  0 et ne 
    // conserver que les 8 bits de poids faible.
    code = code >> 16;
    return code & 0xff;
}

/**
 * --make_code--
 *x
 * Cree notre code
 *
 * @param ascii
 * @param extended
 * @param timeoutIndex
 * @return le code
 */
long make_key_timeout_code( int ascii, int extended, int timeoutIndex )
{
    long code = 0;
    code = ((long)ascii & 0xff) + (((long)extended & 0xff) << 8) + (((long)timeoutIndex & 0xff) << 16);
    return code;
}

/**
 * --getch_timeout2--
 *
 * Renvoie dans un long les infos suivantes
 * -bits 0  7 : Le code ascii de la touche ou 0 s'il s'agit d'une touche tendue
 * -bits 8  15 : 0 s'il s'agit d'une touche non tendue, sinon, le code tendu de la touche
 * -bits 16  23 : le numero de timeout s'il s'agit d'un timeout correspondant a l'index
 * sinon -1
 *
 * @param timeouts
 * @param nbtimeouts
 * @param touches
 * @param nbtouches
 * @return voir plus haut
 */
long getch_timeout2( long timeouts[] , int nbtimeouts, long touches[], int nbtouches)
{
	static clock_t s_tabLastClock[MAX_TIMEOUT] = {0};
	int i;

	// On initialise le tableau la premiere fois
	for( i = 0; i < nbtimeouts; i ++ )
	{
		if( s_tabLastClock[i] == 0 )
		{
			s_tabLastClock[i] = clock();
        }
    }
    
    // On sort uniquement sur timeout ou si la bonne touche est pressee
    while( 1 )
    {
		// On extrait une touche si y'en a une en attente
		int key = 0;
		int ext = 0;

		// Detection appui sur touche
		if( kbhit() != 0 )
		{
			key = getch();
			if( key == 0 || key == 224 )
			{
				key = 0;
				ext = getch();
			}
		}

		// On verifie si la touche est dans le tableau
		for( i = 0; i < nbtouches; i ++ )
		{
			int key_i = code_to_char_ascii( touches[i] );
			int ext_i = code_to_char_extended( touches[i] );
			if( key_i == key && ext_i == ext )
			{
				return make_key_timeout_code( key, ext, -1 );
			}
		}

		// On ne sort que si le timeout est ecoule
		clock_t CurrentClock = clock();
		for( i = 0; i < nbtimeouts; i ++ )
		{
			// On convertit les millisecondes en clock_t
			clock_t clock_timeout = timeouts[i] * CLOCKS_PER_SEC / 1000;
			if( CurrentClock - s_tabLastClock[i] >= clock_timeout )
			{
				// Timeout ecoule
				s_tabLastClock[i] = CurrentClock;
				return make_key_timeout_code( 0, 0, i );
			}
		}

	}
	// Jamais atteint
#ifndef __TURBOC__ // Supprime le warning
	return 0;
#endif
}
