/**
 * @file
 *
 * Main de pacman (mode texte)
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#ifdef _WIN32
#include "coniowin32.h"
#endif
#ifdef __TURBOC__
#include <conio.h>
#endif
#include "txt_tool.h"
#include "pac_core.h"
#include "getch_ex.h"
#include "tab_core.h"
#include "pac_scor.h"

#define	KEY_LEFT	75  /*K*/
#define	KEY_RIGHT	77  /*M*/
#define	KEY_TOP	        72  /*H*/
#define	KEY_BOTTOM	80  /*P*/

#define PACMAN_ARRAY_KEYS_SIZE         14
#define PACMAN_ARRAY_TIMEOUTS_SIZE     2
#define PACTAB_ARRAY_KEYS_SIZE         28

static void pactab();

/**
 * --pacman_make_array_keys()--
 *
 * @return Tableau pour make_array_keys()
 */
static long * pacman_make_array_keys()
{
    static long s_arrayKeys[PACMAN_ARRAY_KEYS_SIZE];
    s_arrayKeys[0] = make_key_timeout_code( 'J', 0, -1 );
    s_arrayKeys[1] = make_key_timeout_code( 'j', 0, -1 );
    s_arrayKeys[2] = make_key_timeout_code( 'Q', 0, -1 );
    s_arrayKeys[3] = make_key_timeout_code( 'q', 0, -1 );
    s_arrayKeys[4] = make_key_timeout_code( 'P', 0, -1 );
    s_arrayKeys[5] = make_key_timeout_code( 'p', 0, -1 );
    s_arrayKeys[6] = make_key_timeout_code( 'S', 0, -1 );
    s_arrayKeys[7] = make_key_timeout_code( 's', 0, -1 );
    s_arrayKeys[8] = make_key_timeout_code( 'T', 0, -1 );
    s_arrayKeys[9] = make_key_timeout_code( 't', 0, -1 );
    s_arrayKeys[10] = make_key_timeout_code( 0, KEY_LEFT, -1 );
    s_arrayKeys[11] = make_key_timeout_code( 0, KEY_RIGHT, -1 );
    s_arrayKeys[12] = make_key_timeout_code( 0, KEY_TOP, -1 );
    s_arrayKeys[13] = make_key_timeout_code( 0, KEY_BOTTOM, -1 );
    return s_arrayKeys;
}

/**
 * --pacman_make_array_timeouts()--
 *
 * @return Tableau pour getch_timeout2()
 */
static long * pacman_make_array_timeouts( int level )
{
    static long s_arrayTimeouts[PACMAN_ARRAY_TIMEOUTS_SIZE];
    switch(level)
    {
    case 0:
    case 1:
        {
            s_arrayTimeouts[0] = 400;
            s_arrayTimeouts[1] = 500;
            break;
        }
    case 2:
        {
            s_arrayTimeouts[0] = 300;
            s_arrayTimeouts[1] = 400;
            break;
        }
    case 3:
        {
            s_arrayTimeouts[0] = 200;
            s_arrayTimeouts[1] = 300;
            break;
        }
    default:
        {
            s_arrayTimeouts[0] = 100;
            s_arrayTimeouts[1] = 200;
            break;
        }
    }
    return s_arrayTimeouts;
}


/**
 * --pacman()
 *
 * jouer a pacman
 */
static void pacman()
{
    int ascii, extended, timeout;
    int play_game = 0;
    int level = 1;

    load_game( "pacman.txt" );
    do {

        // Attente de l'appui sur une touche ou du timeout
        long code = getch_timeout2( 
            pacman_make_array_timeouts(level), PACMAN_ARRAY_TIMEOUTS_SIZE, 
            pacman_make_array_keys(), PACMAN_ARRAY_KEYS_SIZE );
        ascii = toupper(code_to_char_ascii( code ));
        extended = code_to_char_extended( code );
        timeout = code_to_timeout_index( code );

        // Nouvelle partie
        if( ascii == 'J' )
        {
            play_game = 1;
            load_game( "pacman.txt" );
            set_level( 1, 10, 5, 4 );
        }
        // Fin de la partie
        else if( ascii == 'Q' )
        {
            if( toupper( display_message("Quitter O/N ?") ) == 'O' )
            {                
                break;
            }
        }
        // Pause
        else if( ascii == 'P' )
        {
            display_message( "Pause"  );
        }
        else if( ascii == 'S' )
        {
            SCORE aScores[10];
            char lines[11][80]={0};
            char * tabScores[11];
            fill_scores( aScores );
            sprintf( lines[0], "%s\t%s", "NOM", "SCORE" );
            tabScores[0] = lines[0];
            for( int i = 0; i < 10; i ++ )
            {
                if( aScores[i].score > 0 )
                {
                    sprintf( lines[i+1], "%s - %d", aScores[i].name, aScores[i].score );
                }
                tabScores[i+1] = lines[i+1];
            }
            display_messages( tabScores, 11 );
        }
        else if( ascii == 'T' )
        {
            if( ! play_game ) { pactab(); load_game( "pacman.txt" ); }
            else display_message( "Finir la partie avant de changer le tableau..."  );
        }
        else if( extended == KEY_LEFT )
        {
            if( play_game ) go_left();
        }
        else if( extended == KEY_RIGHT )
        {
            if( play_game ) go_right();
        }
        else if( extended == KEY_TOP )
        {
            if( play_game ) go_top();
        }
        else if( extended == KEY_BOTTOM )
        {
            if( play_game ) go_bottom();
        }
        else if( timeout == 0 )
        {
            if( play_game ) move_pacman();
        }
        else if( timeout == 1 )
        {
            if( play_game ) move_monsters();
        }

        // Etat du jeu
        if( play_game && is_game_over() )
        {
            display_message( "Partie perdue!");
            play_game = 0;
	    if( score_top_ten( get_score() ) == 0 )
	    {
		SCORE score;
		
		score.score = get_score();

		get_message( "Nom :", score.name, sizeof(score.name) );
		add_score( score );
            }
        }
        if( play_game && is_level_won() )
        {
            display_message( "Tableau gagne!");
            int lifes = get_lifes();
            load_game( "pacman.txt" );
            set_level( level++, 10, 5, lifes++ );
        }
    } while( 1 );
}


/**
 * --pactab_make_array_keys()--
 *
 * @return Tableau pour make_array_keys()
 */
static long * pactab_make_array_keys()
{
    static long s_arrayKeys[PACTAB_ARRAY_KEYS_SIZE];
    s_arrayKeys[0]  = make_key_timeout_code( 'N', 0, -1 );
    s_arrayKeys[1]  = make_key_timeout_code( 'n', 0, -1 );
    s_arrayKeys[2]  = make_key_timeout_code( 'V', 0, -1 );
    s_arrayKeys[3]  = make_key_timeout_code( 'v', 0, -1 );
    s_arrayKeys[4]  = make_key_timeout_code( 'S', 0, -1 );
    s_arrayKeys[5]  = make_key_timeout_code( 's', 0, -1 );
    s_arrayKeys[6]  = make_key_timeout_code( 'Q', 0, -1 );
    s_arrayKeys[7]  = make_key_timeout_code( 'q', 0, -1 );
    s_arrayKeys[8]  = make_key_timeout_code( '4'/*'?'*/, 0, -1 );
    s_arrayKeys[9]  = make_key_timeout_code( '8'/*'?'*/, 0, -1 );
    s_arrayKeys[10] = make_key_timeout_code( '7'/*'?'*/, 0, -1 );
    s_arrayKeys[11] = make_key_timeout_code( '9'/*'?'*/, 0, -1 );
    s_arrayKeys[12] = make_key_timeout_code( '1'/*'?'*/, 0, -1 );
    s_arrayKeys[13] = make_key_timeout_code( '3'/*'?'*/, 0, -1 );
    s_arrayKeys[14] = make_key_timeout_code( 'W', 0, -1 );
    s_arrayKeys[15] = make_key_timeout_code( 'w', 0, -1 );
    s_arrayKeys[16] = make_key_timeout_code( 'X', 0, -1 );
    s_arrayKeys[17] = make_key_timeout_code( 'x', 0, -1 );
    s_arrayKeys[18] = make_key_timeout_code( ' ', 0, -1 );
    s_arrayKeys[19] = make_key_timeout_code( 'O', 0, -1 );
    s_arrayKeys[20] = make_key_timeout_code( 'o', 0, -1 );
    s_arrayKeys[21] = make_key_timeout_code( 'A', 0, -1 );
    s_arrayKeys[22] = make_key_timeout_code( 'a', 0, -1 );
    s_arrayKeys[23] = make_key_timeout_code( ' ', 0, -1 );
    s_arrayKeys[24] = make_key_timeout_code( 0, KEY_LEFT, -1 );
    s_arrayKeys[25] = make_key_timeout_code( 0, KEY_RIGHT, -1 );
    s_arrayKeys[26] = make_key_timeout_code( 0, KEY_TOP, -1 );
    s_arrayKeys[27] = make_key_timeout_code( 0, KEY_BOTTOM, -1 );
    return s_arrayKeys;
}

/**
 * --pacman()
 *
 * creer les tableaux de pacman
 */
static void pactab()
{
    int ascii, extended, timeout;

    int x = 27;
    int y = 21;
    tab_new( x, y );

    do {

        // Attente de l'appui sur une touche ou du timeout
        long code = getch_timeout2( 
            NULL, 0, 
            pactab_make_array_keys(), PACTAB_ARRAY_KEYS_SIZE );
        ascii = toupper(code_to_char_ascii( code ));
        extended = code_to_char_extended( code );

        // Nouveau tableau
        if( ascii == 'N' )
        {
            char szBuffer[20] = {0};
            int x = 27;
            int y = 21;
            get_message( "Taille du tableau en X (27 par default) : ", szBuffer, sizeof(szBuffer) );
            x = atoi(szBuffer); if (x < 0 || x > 70 ) x = 27;
            get_message( "Taille du tableau en Y (21 par default) : ", szBuffer, sizeof(szBuffer) );
            y = atoi(szBuffer); if (y < 0 || y > 50 ) y = 21;
            tab_new( x, y );
        }

        // Ouvrir tableau
        else if( ascii == 'V' )
        {
            char szFile[50] = "pacman.txt";
            get_message( "Charger fichier : ", szFile, sizeof(szFile) );
            tab_open( szFile );
        }

        // Sauvegarder tableau
        else if( ascii == 'S' )
        {
            char szFile[50] = "pacman.txt";
            get_message( "Sauver fichier : ", szFile, sizeof(szFile) );
            if( strstr( szFile, "pacman.txt") )
            {
                if( toupper( display_message("Ecraser pacman.txt O/N ?") ) == 'O' )
                {                
                    tab_save( szFile );
                }
            }
            else
            {
                tab_save( szFile );
            }
        }

        // Quitter
        else if( ascii == 'Q' )
        {
            if( toupper( display_message("Quitter O/N ?") ) == 'O' )
            {                
                break;
            }
        }
        else if( ascii == '4'/*'' 179*/ )
        {
            tab_wall_vert();
        }
        else if( ascii == '8'/*'' 196*/ )
        {
            tab_wall_horiz();
        }
        else if( ascii == '7'/*'' 218*/ )
        {
            tab_wall_top_left();
        }
        else if( ascii == '9'/*'' 191*/ )
        {
            tab_wall_top_right();
        }
        else if( ascii == '1'/*'' 192*/ )
        {
            tab_wall_bottom_left();
        }
        else if( ascii == '3'/*'' 217*/ )
        {
            tab_wall_bottom_right();
        }
        else if( ascii == 'W' )
        {
            tab_fruit();
        }
        else if( ascii == 'X' )
        {
            tab_magic_fruit();
        }
        else if( ascii == 'O' )
        {
            tab_pacman();
        }
        else if( ascii == 'A' )
        {
            tab_monster();
        }
        else if( ascii == ' ' )
        {
            tab_nothing();
        }
        //else if( ascii == ' ' )
        //{
        //    tab_repeat();
        //}
        else if( extended == KEY_LEFT )
        {
            tab_left();
        }
        else if( extended == KEY_RIGHT )
        {
            tab_right();
        }
        else if( extended == KEY_TOP )
        {
            tab_top();
        }
        else if( extended == KEY_BOTTOM )
        {
            tab_bottom();
        }

    } while( 1 );
}

/**
 * --main()
 *
 * Le main() de pacman
 */
int main()
{
    //pactab();
    pacman();
    return 0;
}


