/**
 * @file
 *
 * Gestion des 10 meilleurs scores (cp from tetris)
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pac_scor.h"

// Variable globale pour stocker les scores aprs lecture
// du fichier et avant criture du fichier
static SCORE s_aScores[10];


/**
 * --write_scores()--
 *
 * Ecrit le fichier de score a partir de la structure
 *
 * @return 0 OK, -1 erreur
 */
static int write_scores()
{
    FILE * fileScore;
    int iLigne;

    fileScore = fopen( "scores.txt", "w" );
    if( fileScore == NULL )
    {
	return -1;
    }

    iLigne = 0;
    while( s_aScores[iLigne].score != 0 && iLigne < 10 )
    {
	fprintf( fileScore, "%10d %s\n", s_aScores[iLigne].score, s_aScores[iLigne].name );
	iLigne ++;
    }
    fclose( fileScore );
    return 0;

}

/**
 * --read_scores()--
 *
 * Remplit la structure s_aScores  partir du fichier de score
 *
 * @return 0 OK, -1 erreur
 */
static int read_scores()
{
    FILE * fileScore;
    char szLigne[200]; int iLigne;

    fileScore = fopen( "scores.txt", "r" );
    if( fileScore == NULL )
    {
        return -1;
    }
    iLigne = 0;
    while( fgets( szLigne, sizeof(szLigne), fileScore ) && iLigne < 10 )
    {
	// On vire le \n de fin
	int n = (int)strlen(szLigne);
	if( szLigne[n-1] == '\n' || szLigne[n-1] == '\r') szLigne[n-1] = 0;
	if( szLigne[n-2] == '\n' || szLigne[n-2] == '\r') szLigne[n-2] = 0;
	
	// On met un 0 bien plac pour rcuprer le nombre de lignes    
	szLigne[10] = 0;
	s_aScores[iLigne].score = atoi( szLigne );
	strncpy(  s_aScores[iLigne].name, szLigne + 11, sizeof(s_aScores[iLigne].name) - 1 );
	iLigne ++;
    }
    fclose( fileScore );
    return 0;
}


/**
 * --score_top_ten()--
 *
 * @param score score a tester
 * @return 0 si le score fait parti des 10 meilleurs
 */
int score_top_ten( int score )
{
    if( read_scores() == -1 ) return 0;
    for( int i = 0; i < 10; i ++ )
    {
        if( score > s_aScores[i].score )
        {
	    return 0;
        }
    }
    return -1;
}


/**
 * --fill_scores()--
 *
 * Remplit le tableau aScores[10] avec les 10 meilleurs scores.
 * S'il y a moins de 10 scores, les dernires valeurs sont 
 * initialises  0
 *
 * @param aScores tableau de scores
 */
void fill_scores( SCORE aScores[10] )
{
    memset( aScores, 0, sizeof(SCORE) * 10 );
    if( read_scores() == -1 ) return;
    memcpy( aScores, s_aScores, sizeof(SCORE) * 10 );
}


/**
 * --add_score()--
 *
 * Ajoute un score dans le fichier. On fait en sorte d'ajouter
 * le score  la bonne position dans le fichier
 *
 * @param score le score
 */
void add_score( SCORE score )
{
    int i;
    // Remplit le tableau
    read_scores();

    // On cherche ou on doit se mettre
    for( i = 0; i < 10; i ++ )
    {
        if( score.score > s_aScores[i].score )
        {
            break;
        }
    }

    // On dcale toutes les valeurs aprs
    for( int j = 9; j > i; j -- )
    {
	s_aScores[j] = s_aScores[j-1];
    }

    // On met le score
    s_aScores[i] = score;

    // On met  jour le fichier
    write_scores();
}
