/**
 * @file
 *
 * Routines d'affichage en mode texte pour pacman
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <conio.h>
#ifdef _WIN32
#include "coniowin32.h"
#endif
#ifdef __TURBOC__
#include <conio.h>
#define putch2 putch
#endif
#include "txt_tool.h"
#include "pac_core.h"
#include "pac_view.h"

/**
 * Position de l'aire de jeu a l'ecran
 */
static int s_xGameTop;
static int s_yGameTop;
static int s_xGameBottom;
static int s_yGameBottom;

/**
 * Position des infos a l'ecran
 */
static int s_xInfosTop;
static int s_yInfosTop;
static int s_xInfosBottom;
static int s_yInfosBottom;

/**
 * --x_pac_to_x_screen()--
 *
 * Convertit un x du jeu en x a afficher
 *
 * @param x x du jeu
 * @return le x ecran
 */
static int x_pac_to_x_screen( int x )
{
    return x + s_xGameTop + 1;
}

/**
 * --y_pac_to_y_screen()--
 *
 * Convertit un y du jeu en y a afficher
 *
 * @param y y du jeu
 * @return le y ecran
 */
static int y_pac_to_y_screen( int y )
{
    return y + s_yGameTop + 1;
}

/**
 * --display_screen()--
 *
 * Reaffiche tout l'ecran
 */
void display_screen()
{
    static int initialized = 0;
    if( initialized == 0 )
    {
        init_txt_mode();
    }
    
    // Calcule la zone de jeu
    s_xGameTop = ( get_max_cols() - (get_x_size() + 2) ) / 2;
    s_yGameTop = ( get_max_rows() - (get_y_size() + 2) ) / 2 - 3;
    s_xGameBottom = s_xGameTop + get_x_size() + 1;
    s_yGameBottom = s_yGameTop + get_y_size() + 1;

    // Calcule la zone des infos
    s_xInfosTop = s_xGameTop;
    s_yInfosTop = s_yGameBottom + 2;
    s_xInfosBottom = s_xGameBottom;
    s_yInfosBottom = s_yInfosTop + 2;

    // Affiche le fond
    set_screen_background( CYAN, 0xB0 );

    // Affiche le menu
    char shorLetter[] = { 'J', 'Q', 'P', 'S', 'T', 27, 26, 24, 25 };
    char * menus[] = { "Jeu", "Quitter", "Pause", "Scores", "Tableaux", "Gauche", "Droite", "Haut", "Bas" };
    display_status_bar( shorLetter, menus, sizeof(shorLetter)/sizeof(char) );

    // Affiche l'aire de jeu
    delete_screen_zone( s_xGameTop, s_yGameTop, s_xGameBottom, s_yGameBottom );
    display_box( s_xGameTop, s_yGameTop, s_xGameBottom, s_yGameBottom );
}

/**
 * --display_item()--
 *
 * Affiche un element (pacman, mur, monstre, fruit, rien)
 *
 * @param x position de l'element en x
 * @param y position de l'element en y
 * @param type type de l'element
 * @param pacEater 0 , 1, ou 2
 */
void display_item( int x, int y, int type, int pacEater )
{
    int xScreen, yScreen;
    int backColor = BLACK;
    int textColor = WHITE;
    int c = ' ' ;

    // Conversion du type en caractere affichable
    type_to_char_and_color( type, pacEater, &backColor, &textColor, &c );
    
    // Conversion
    xScreen = x_pac_to_x_screen( x );
    yScreen = y_pac_to_y_screen( y );
    gotoxy( xScreen, yScreen );

    // Afficher le char
    textcolor( textColor );
    textbackground( backColor );
    putch2( c );
}

/**
 * --display_infos()--
 *
 * Affiche les infos classiques : Score, nombre de vie
 */
void display_infos()
{
    // Affiche l'aire des infos
    delete_screen_zone( s_xInfosTop, s_yInfosTop, s_xInfosBottom, s_yInfosBottom );
    display_box( s_xInfosTop, s_yInfosTop, s_xInfosBottom, s_yInfosBottom );

    // Affiche les infos
    gotoxy( s_xInfosTop + 1, s_yInfosTop + 1 );
    cprintf( "Score:%d - Vies:%1d - Niv:%1d", get_score(), get_lifes(), get_level() );
}

/**
 * --type_to_char_and_color()--
 *
 * Donne le type a afficher en fonction du type interne
 *
 * @param type type de l'element
 * @param pacEater 0 , 1, ou 2
 * @param pbackColor La couleur de fond en sortie
 * @param ptextColor La couleur de texte en sortie
 * @param pc le caractere a afficher en sortie
 */
void type_to_char_and_color( int type, int pacEater, int * pbackColor, int * ptextColor, int *pc )
{
    *pbackColor = BLACK;
    *ptextColor = WHITE;
    *pc = ' ' ;
    switch( type )
    {
    case PAC_TYPE_NOTHING:
        {
            *pc = CHAR_PAC_SPACE;
            break;
        }
    case PAC_TYPE_WALL_VERT:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_WALL_HORIZ:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_WALL_TOP_LEFT:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_WALL_TOP_RIGHT:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_WALL_BOTTOM_LEFT:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_WALL_BOTTOM_RIGHT:
        {
            *pc = CHAR_PAC_WALL;
            break;
        }
    case PAC_TYPE_PACMAN:
        {
            *pc = CHAR_PAC_PACMAN;
            if( pacEater == 1 || pacEater == 2 ) *ptextColor = RED;
            else *ptextColor = GREEN;
            break;
        }
    case PAC_TYPE_MONSTER:
        {
            *pc = CHAR_PAC_MONSTER;
            if( pacEater == 1 )
            {
                *ptextColor = GREEN;
            }
            else if( pacEater == 2 ) 
            {
#ifdef BLINK
                *ptextColor = RED + BLINK;
#else
                *ptextColor = BLUE;
#endif
            }
            else
            {
                *ptextColor = RED;
            }
            break;
        }
    case PAC_TYPE_FRUIT:
        {
            *pc = CHAR_PAC_FRUIT;
            break;
        }
    case PAC_TYPE_MAGIC_FRUIT:
        {
            *pc = CHAR_PAC_MAGIC_FRUIT;
            *ptextColor = YELLOW;
            break;
        }
    }
}
