/**
 * @file
 *
 * Routines d'affichage en mode texte pour la creation de tableaux
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <conio.h>
#ifdef _WIN32
#include "coniowin32.h"
#endif
#ifdef __TURBOC__
#include <conio.h>
#define putch2 putch
#endif
#include "txt_tool.h"
#include "tab_core.h"
#include "pac_core.h"
#include "tab_view.h"
#include "pac_view.h"

/**
 * Position de l'aire de jeu a l'ecran
 */
static int s_xGameTop;
static int s_yGameTop;
static int s_xGameBottom;
static int s_yGameBottom;


/**
 * --x_pac_to_x_screen()--
 *
 * Convertit un x du jeu en x a afficher
 *
 * @param x x du jeu
 * @return le x ecran
 */
static int x_pac_to_x_screen( int x )
{
    return x + s_xGameTop + 1;
}

/**
 * --y_pac_to_y_screen()--
 *
 * Convertit un y du jeu en y a afficher
 *
 * @param y y du jeu
 * @return le y ecran
 */
static int y_pac_to_y_screen( int y )
{
    return y + s_yGameTop + 1;
}

/**
 * --tab_display_screen()--
 *
 * Reaffiche tout l'ecran
 */
void tab_display_screen()
{
    static int initialized = 0;
    if( initialized == 0 )
    {
        init_txt_mode();
        // Curseur normal
        _setcursortype( _NORMALCURSOR );
    }
    
    // Calcule la zone de jeu
    s_xGameTop = ( get_max_cols() - (tab_get_x_size() + 2) ) / 2;
    s_yGameTop = ( get_max_rows() - (tab_get_y_size() + 2) ) / 2 - 3;
    s_xGameBottom = s_xGameTop + tab_get_x_size() + 1;
    s_yGameBottom = s_yGameTop + tab_get_y_size() + 1;

    // Affiche le fond
    set_screen_background( GREEN, 176 );


    // Affiche le menu
    char shorLetter[] = { 
        'N', 'V', 'S', 'Q', 
        '4', 
        '8', 
        '7', 
        '9', 
        '1', 
        '3',
        'W', 'X',
        'A', 'O',
        27, 26, 24, 25 };
    char * menus[] = { 
        "No", "ouV", "Sav", "Qu", 
        /*
        STRING_WALL_VERT,
        STRING_WALL_HORIZ,
        STRING_WALL_TOP_LEFT,
        STRING_WALL_TOP_RIGHT,
        STRING_WALL_BOTTOM_LEFT,
        STRING_WALL_BOTTOM_RIGHT,
        */
        "",
        "",
        "",
        "",
        "",
        "",
        "Fr", "Fr2",
        "", "",
        "", "", "", "" };
    display_status_bar( shorLetter, menus, sizeof(shorLetter)/sizeof(char) );

    // Affiche l'aire de jeu
    delete_screen_zone( s_xGameTop, s_yGameTop, s_xGameBottom, s_yGameBottom );
    display_box( s_xGameTop, s_yGameTop, s_xGameBottom, s_yGameBottom );
}

/**
 * --tab_display_item()--
 *
 * Affiche un element (pacman, mur, monstre, fruit, rien)
 *
 * @param x position de l'element en x
 * @param y position de l'element en y
 * @param type type de l'element
 */
void tab_display_item( int x, int y, int type )
{
    int xScreen, yScreen;
    int backColor = BLACK;
    int textColor = WHITE;
    int c = ' ' ;

    // Conversion du type en caractere affichable
    type_to_char_and_color( type, 0, &backColor, &textColor, &c );

    // Conversion
    xScreen = x_pac_to_x_screen( x );
    yScreen = y_pac_to_y_screen( y );
    gotoxy( xScreen, yScreen );

    // Afficher le char
    textcolor( textColor );
    textbackground( backColor );
    putch2( c );
}
