/**
 * @file
 *
 * Routines d'affichage en mode texte
 * On considere que l'ecran a 80 colonnes et 50 lignes
 *
 * @author Sylvain MARECHAL
 * @date $Id$
 */
#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#ifdef _WIN32
#include "coniowin32.h"
#endif
#ifdef __TURBOC__
#define putch2 putch
#endif
#include "txt_tool.h"
#define CHAR_BOX_WALL   0xB2
#define CHAR_BOX_SPACE  0x20

/**
 * --get_max_cols()--
 *
 * @return le nombre de colonnes
 */
int get_max_cols()
{
    return 80;
}

/**
 * --get_max_rows()--
 *
 * @return le nombre de lignes
 */
int get_max_rows()
{
    return 50;
}


/**
 * --init_txt_mode()--
 *
 * Initialisation de l'affichage en mode 80/50
 * Suppression du curseur
 * Effacement de l'cran
 * Fonction  appeler en dbut de programme
 */
void init_txt_mode()
{
    // Passage en mode 50 lignes
    textmode( C4350 );

    // Curseur transparent
    _setcursortype( _NOCURSOR );
    
    // Efface l'cran
    clrscr();
}

/**
 * --restore_last_txt_mode()--
 *
 * A appeler en fin de programme
 */
void restore_last_txt_mode()
{
    // Ancien mode
    textmode( LASTMODE );

    // Curseur normal
    _setcursortype( _NORMALCURSOR );
    
    // Efface l'cran
    clrscr();
}

/**
 * --set_screen_background()--
 *
 * @param backColor couleur de fond (ex CYAN)
 * @param caractere (ex:176)
 */
void set_screen_background( int backColor, int ch )
{
    //textcolor( WHITE );
    textbackground( backColor );
    for( int i = 1; i < get_max_rows(); i ++ )
    {
	for( int j = 1; j <= get_max_cols(); j ++ )
	{
	    gotoxy( j, i );
	    putch2( ch );
	}
    }
}


/**
 * --status_put_char_and_string()--
 *
 * Affichage d'un element du menu de la barre de status
 * a la position du curseur
 *
 * @param c La lettre de menu
 * @param string le libelle
 */
static void status_put_char_and_string( char c, char * string )
{
    textbackground( LIGHTGRAY );
    textcolor( RED );
    putch2( CHAR_BOX_SPACE );
    putch2( c );
    putch2( CHAR_BOX_SPACE );
    textcolor( BLACK );
    for( int i = 0; i < (int )strlen(string); i ++ )
    {
	putch2( string[i] );
    }
}


/**
 * --delete_status_bar()--
 *
 * Efface la barre de status
 */
static void delete_status_bar()
{
    gotoxy( 1, get_max_rows() );
    textbackground( LIGHTGRAY  );
    for( int i = 1; i < get_max_cols(); i ++ ) 
	putch2( CHAR_BOX_SPACE );
}


/**
 * --display_status_bar()--
 *
 * Affiche une barre de menu
 *
 * @param shorLetter tableau avec les lettres de racourci
 * @param menus tableau de libelles de menus
 * @param cbMenus nombre de menus
 */
void display_status_bar( char shorLetter[], char * menus[], int cbMenus )
{
    delete_status_bar();
    gotoxy( 1, get_max_rows() );
    for( int i = 0; i < cbMenus; i ++ )
    {
        status_put_char_and_string( shorLetter[i], menus[i] );
    }
}

/**
 * --display_box()--
 *
 * Affiche un tour avec le caractre ascii 178.
 *
 * @param colonneDeb
 * @param ligneDeb
 * @param colonneFin
 * @param ligneFin
 */
void display_box( int colonneDeb, int ligneDeb, int colonneFin, int ligneFin)
{
    // Couleur du tour
    textbackground( LIGHTGRAY );
    textcolor( BLACK );

    // On affiche le tour
    for( int i = ligneDeb; i <= ligneFin; i ++ )
    {
	gotoxy( colonneDeb, i );
	putch2( CHAR_BOX_WALL );
	gotoxy( colonneFin, i );
	putch2( CHAR_BOX_WALL );
    }
    for( int j = colonneDeb; j <= colonneFin; j ++ )
    {
	gotoxy( j, ligneDeb );
	putch2( CHAR_BOX_WALL );
	gotoxy( j, ligneFin );
	putch2( CHAR_BOX_WALL );
    }
}


/**
 * --delete_screen_zone()--
 *
 * Efface une partie de l'cran en mettant des blancs.
 * La combinaison window()/clrscr() n'est pas utilise ici
 * car elle n'a pas t porte sous WIN32
 *
 * @param colonneDeb
 * @param ligneDeb
 * @param colonneFin
 * @param ligneFin
 */
void delete_screen_zone( int colonneDeb, int ligneDeb, int colonneFin , int ligneFin)
{
    for( int i = ligneDeb; i <= ligneFin; i ++ )
    {
	for( int j = colonneDeb; j <= colonneFin; j ++ )
	{
	    gotoxy( j, i );
	    putch2( CHAR_BOX_SPACE );
	}
    }
}


/**
 * --display_message()--
 *
 * Affiche un message
 *
 * @param pszMessage Le message a afficher
 * @return la touche sur laquelle on a appuye
 */
int display_message( char * pszMessage )
{
    int len = (int)strlen(pszMessage) + 2;
    int left = (get_max_cols() - len)/2;
    int top = (get_max_rows() - 3)/2;
    int right = left + len + 1;
    int bottom = top + 1 + 1;
    char BufferSave[1000];
    int key;
    
    // Sauvegarde de l'cran
    gettext( left, top, right, bottom, BufferSave );
    
    // Affichage du message
    textbackground( WHITE );
    textcolor( RED );
    delete_screen_zone( left, top, right, bottom );
    display_box( left, top, right, bottom );
    gotoxy( left + 1, top + 1 );
    cprintf( pszMessage );

    // Attend touche
    /*while( 1 )
    {
	int touche = toupper(getch());
	if( touche == ' ' || touche == 'O' || touche == 'N' ) break;
    }*/
    key = getche();

    // Restauration cran
    puttext( left, top, right, bottom, BufferSave );
    return key;
}

/**
 * --display_messages()--
 *
 * Affiche une liste de message
 *
 * @param apszMessage Le tableau
 * @param cb La taille du tableau
 * @return la touche sur laquelle on a appuye
 */
int display_messages( char * apszMessage[], int cb )
{
    int len = 0;
    int i;
    for( i = 0; i < cb; i ++ )
    {
        if( len < (int)strlen(apszMessage[i]) + 2 )
        {
            len = (int)strlen(apszMessage[i]) + 2;
        }
    }
    int left = (get_max_cols() - len)/2;
    int top = (get_max_rows() - (cb+2))/2;
    int right = left + len + 1;
    int bottom = top + cb + 1;
    char BufferSave[1000];
    int key;
    
    // Sauvegarde de l'cran
    gettext( left, top, right, bottom, BufferSave );
    
    // Affichage du message
    textbackground( WHITE );
    textcolor( RED );
    delete_screen_zone( left, top, right, bottom );
    display_box( left, top, right, bottom );
    for( i = 0; i < cb; i ++ )
    {
        gotoxy( left + 1, top + 1+i );
        cprintf( apszMessage[i] );
    }

    key = getch();

    // Restauration cran
    puttext( left, top, right, bottom, BufferSave );
    return key;
}
/**
 * --get_message()--
 *
 * Saisie d'un message
 *
 * @param pszTitre titre de la boite
 * @param pszMessage message a saisir 
 * @param maxMessage longueur maxi
 */
void get_message( const char * pszTitre, char * pszMessage, int maxMessage )
{
    int len = maxMessage + (int)strlen(pszTitre);
    int left = (get_max_cols() - len)/2;
    int top = (get_max_rows() - 3)/2;
    int right = left + len + 1;
    int bottom = top + 1 + 1;
    char BufferSave[1000];
    
    // Sauvegarde de l'cran
    gettext( left, top, right, bottom, BufferSave );
    
    // Saisie du message
    textbackground( WHITE );
    textcolor( RED );
    delete_screen_zone( left, top, right, bottom );
    display_box( left, top, right, bottom );
    gotoxy( left + 1, top + 1 );
    cprintf( pszTitre );
    fgets( pszMessage, maxMessage, stdin );
    char * cr = strchr(pszMessage, '\r');
    if( cr ) *cr = 0;
    cr = strchr(pszMessage, '\n');
    if( cr ) *cr = 0;

    // Restauration cran
    puttext( left, top, right, bottom, BufferSave );
}
