#ifdef _WIN32
#pragma warning(disable:4996) /*Suppression du warning ""'' was declared deprecated""*/
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pac_core.h"
#include "pac_err.h"
#include "pac_file.h"

/**
 * --read_line()--
 *
 * Lit une ligne de fichier de type txt1;txt2;txt3;
 * et renvoie dans data1 txt1, data2 txt2 et data3 txt3. 
 * La ligne ne doit pas exceder 256 caracteres et les buffers
 * data1,2,3 doivent tre assez grands
 *
 * @param file  Handle de fichier
 * @param data1
 * @param data2
 * @param data3
 * @return 0 si tout s'est bien passe, sinon, -1
 */
static int read_line( FILE * file, char * data1, char * data2, char * data3 )
{
    char buf[256];
    int ret = -1;
    if( fgets( buf, sizeof(buf), file ) != NULL )
    {
        char * p = strchr( buf, ';' );
        if( p )
        {
            *p = 0;
            strcpy( data1, buf );
            char * p2 = strchr( p+1, ';' );
            if( p2 )
            {
                *p2 = 0;
                strcpy( data2, p+1 );
                
                p = strchr( p2+1, ';' );
                if( p )
                {
                    *p = 0;
                    strcpy( data3, p2 + 1 );
                    ret = 0;
                }
            }
        }
    }
    return ret;
}

/**
 *  --load_file()--
 *
 * Charge un tableau a partir d'un fichier et initialise
 * toutes les variables.
 * Si un probleme se produit, on quitte le jeu
 *
 * @param gameFile Le nom du fichier
 */
void load_file( const char * gameFile, int * pxSize, int * pySize, int ** parrayGame )
{
    FILE * file;
    char data1[256];
    char data2[256];
    char data3[256];
    int xSize = 0, ySize = 0;
    int * arrayGame = NULL;

    // Ouverture du fichier
    file = fopen( gameFile, "rt" );
    if( file == NULL )
    {
        error_exit( "Ouverture du fichier tableau impossible !");
    }
    // La premiere ligne du fichier fait office d'entete et contient:
    // un identificateur "PACMAN" suivi de la taille du tableau
    // Ex: PACMAM;27;21;
    if( read_line( file, data1, data2, data3 ) == -1 )
    {
        error_exit( "Fichier tableau non conforme (1) !");
    }

    // Exploitation des donnees de l'entete
    if( strcmp( data1, "PACMAN"  ) )
    {
        error_exit( "Fichier tableau non conforme (2) !");
    }    

    // Taille du tableau en cases
    xSize = atoi(data2);
    ySize = atoi(data3);
    if( xSize <= 0 || ySize <= 0 )
    {
        error_exit( "Fichier tableau non conforme (3) !");
    }

    // Allocation du tableau de jeu
    arrayGame = (int *)calloc( xSize * ySize * sizeof(int), 1 );
    if( arrayGame == NULL )
    {
        error_exit( "Allocation memoire impossible (1) !");
    }

    
    // Chaque ligne de fichier contient 3 donnees au format texte: 
    // Le type (enum PAC_TYPE), la position x, la position y
    // les donnees sont separees par des ';'
    // Ex: 1;10;15 (On a un mur (WALL=1) pour x=10 et y=15
    // Les lignes non definies sont mises a NOTHING (0)
    do
    {
        if( read_line( file, data1, data2, data3 ) == 0 )
        {
            int type = atoi(data1);
            int x = atoi(data2);
            int y = atoi(data3);
            arrayGame[(y*xSize) + x] = type;
        }
    } while( ! feof(file) );

    // On ferme le fichier
    fclose( file );

    // On met a jour les variables pour l'appelant
    *pxSize = xSize;
    *pySize = ySize;
    *parrayGame = arrayGame;
}

/**
 *  --save_file()--
 *
 */
void save_file( const char * pszFile, int xSize, int ySize, int * arrayGame )
{
    FILE * file = fopen( pszFile, "wt" );
    if( file == NULL )
    {
        error_exit( "Ecriture du fichier tableau impossible !");
    }
    // La premiere ligne du fichier fait office d'entete et contient:
    // un identificateur "PACMAN" suivi de la taille du tableau
    // Ex: PACMAM;27;21;
    fprintf( file, "%s;%d;%d;\n", "PACMAN", xSize, ySize );
    for( int x = 0; x < xSize; x ++ )
    {
        for( int y = 0; y < ySize; y ++ )
        {
            fprintf( file, "%d;%d;%d;\n", arrayGame[(y*xSize) + x], x, y );
        }
    }
    fclose(file);
}
